// Bug report for macOS: QScrollBar gradually slowing down paint
// performance when using the Mac style
// 28 April 2021

#include <QApplication>

#include <QMainWindow>
#include <QTableWidget>
#include <QScrollBar>
#include <QHeaderView>

#include <QTime>
#include <QTimer>
#include <QElapsedTimer>

#include <QRandomGenerator>
#include <QDebug>

const int NUMBER_OF_ROWS = 300;
const int NUMBER_OF_COLS = 30;
const int SCROLL_TIMER_INTERVAL = 10;
const int NUMBER_OF_PAINTS_TO_AVERAGE = 20;



class TableWidget : public QTableWidget {
	Q_OBJECT
	QElapsedTimer lifeTimer;
	QElapsedTimer paintTimer;
	int paintCount = 0;
	int paintSum = 0;

	void paintEvent(QPaintEvent* event) override {
		//display average time for last few paints
		paintCount++;
		paintTimer.start();

		QTableWidget::paintEvent(event);
		paintSum = paintSum + paintTimer.elapsed();

		if (paintCount >= NUMBER_OF_PAINTS_TO_AVERAGE) {
			float averageTime = float(paintSum) / NUMBER_OF_PAINTS_TO_AVERAGE;
			QString timeStamp = QTime::fromMSecsSinceStartOfDay(int(lifeTimer.elapsed())).toString("hh:mm:ss.zzz");

			qDebug().noquote()
					<< timeStamp
					<< "Avg paint time ("
					<< NUMBER_OF_PAINTS_TO_AVERAGE
					<< "): "
					<< averageTime
					<< "ms";

			paintSum = 0;
			paintCount = 0;
		}

	}

public:
	TableWidget(QWidget* parent=nullptr) : QTableWidget(parent) {
		lifeTimer.start();

		setRowCount(NUMBER_OF_ROWS);
		setColumnCount(NUMBER_OF_COLS);
		verticalHeader()->setDefaultSectionSize(8);

		// fill widget with a multiplication table to give the painter more to do
		for (int r = 0; r < NUMBER_OF_ROWS; r++) {
			for (int c = 0; c < NUMBER_OF_COLS; c++) {
				setItem(r, c, new QTableWidgetItem(QString::number(r*(c+1))));
			}
		}
	}
};


class MainWindow : public QMainWindow {
	Q_OBJECT

	QTimer scrollTimer;
	TableWidget* tableWidget = nullptr;

public:
	MainWindow(QWidget* parent = nullptr) : QMainWindow(parent) {

		tableWidget = new TableWidget(this);
		this->setCentralWidget(tableWidget);

		scrollTimer.setInterval(SCROLL_TIMER_INTERVAL);
		connect(&scrollTimer, &QTimer::timeout, this, &MainWindow::tick);
		scrollTimer.start();

		if (qApp->arguments().contains("-w")) {
			tableWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
			tableWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
		}

		this->setMinimumSize(300, 300);
		this->setWindowTitle("QT " QT_VERSION_STR " bug report");
	}

	~MainWindow() { }

private slots:
	void tick() {
		int row = QRandomGenerator::global()->bounded(0, NUMBER_OF_ROWS);
		int col = QRandomGenerator::global()->bounded(0, NUMBER_OF_COLS);

		tableWidget->scrollToItem(tableWidget->item(row,col));
	}

};

int main(int argc, char *argv[])
{
	QApplication app(argc, argv);
	MainWindow mw;

	mw.show();
	return app.exec();
}

#include  "main.moc"
